incsrc subroutinedefs.asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SMW Sumo Bro Lightning (sprite 2B), by imamelia
;;
;; This is a disassembly of sprite 2B in SMW, the lightning that a Sumo Brother
;; spawns that bursts into flame upon contact with ground.
;;
;; Uses first extra bit: NO
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines and tables
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Tilemap:
db $04,$2B

FlameXOffsetLo:
db $FC,$0C,$EC,$1C,$DC

FlameXOffsetHi:
db $FF,$00,$FF,$00,$FF

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
PHB
PHK
PLB
JSR SumoLightningMain
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SumoLightningMain:

LDA $1540,x	; if the timer for showing flames is set...
BNE ShowFlames	; show the flames

LDA #$30		;
STA $AA,x	; set the sprite Y speed

JSL $81801A	; update sprite Y position without gravity

LDA $1FE2,x	; if the object contact disable timer is set...
BNE NoObjCont	; don't check contact with objects

JSL $819138	; object interaction routine

LDA $1588,x	;
AND #$04	; if the sprite has hit the ground...
BEQ NoObjCont	;

LDA #$17		; play the fire sound effect
STA $1DFC	;
LDA #$22		; set the time to show flames before the sprite disappears
STA $1540,x	;

LDA $15A0,x	;
ORA $186C,x	; unless the sprite is offscreen..
BNE NoObjCont	;
LDA $E4,x	;
STA $9A		; set the sprite position to the block base position
LDA $D8,x	;
STA $98		;

JSL $828A44	; generate a puff of smoke

NoObjCont:

JSR SumoLightningGFX	; draw the sprite

Return00:			;
RTS			;


ShowFlames:

STA $02		;
CMP #$01		; if the timer is about to run out...
BNE NoEraseSpr	;
STZ $14C8,x	; erase the sprite
NoEraseSpr:	;
AND #$0F	;
CMP #$01		;
BNE Return00	;
STA $18B8	; activate cluster sprites

JSR GenerateFlames	; generate the group of 10 cluster sprites that form the flames

INC $1570,x	;
LDA $1570,x	;
CMP #$01		;
BEQ NoSecondGen	;

JSR GenerateFlames	;

INC $1570,x	;

NoSecondGen:	;
RTS		;

GenerateFlames:

LDA $E4,x	;
STA $00		;
LDA $14E0,x	; backup the sprite X position
STA $01		;

LDY #$09		; 10, or 0x0A, cluster sprite slots to loop through

FindFreeClusterSlot:

LDA $1892,y		; if this cluster sprite slot is empty...
BEQ SpawnFlame		; spawn a flame here
DEY			; if not...
BPL FindFreeClusterSlot	; check the next index

DEC $191D		; $191D: UNKNOWN RAM ADDRESS
BPL NoResetIndex		; if it has dropped below 0...
LDA #$09			; reset it to 9
STA $191D		;
NoResetIndex:		;
LDY $191D		;

SpawnFlame:	;

PHX		;
LDA $1570,x	;
TAX		;

LDA $00			;
CLC			;
ADC FlameXOffsetLo,x	; set the X position of the flame relative to the lightning
STA $1E16,y		;
LDA $01			;
ADC FlameXOffsetHi,x	;
STA $1E3E,y		;

PLX		;

LDA $D8,x	;
SEC		;
SBC #$10		; the Y offset is always -$10
STA $1E02,y	;
LDA $14D4,x	; set the Y position of the flame
SBC #$00		;
STA $1E2A,y	;

LDA #$7F		; time to show the flame?
STA $0F4A,y	;

LDA $1E16,y	;
CMP $1A		;
LDA $1E3E,y	;
SBC $1B		; if the cluster sprite is offscreen...
BNE NoGenFlame	; don't generate it

LDA #$06		; set the cluster sprite number
STA $1892,y	; 06 = Sumo Bro flame

NoGenFlame:	;
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; graphics routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SumoLightningGFX:

JSL !GetDrawInfo	;

LDA $00		;
STA $0300,y	;

LDA $01
STA $0301,y ; The Y position code, which is above our one.

PHX
LDA $14
LSR #3
AND #$01
TAX
LDA Tilemap,x	;
STA $0302,y	;
PLX		
LDA $15F6,x	;
;ORA $64		;
STA $0303,y	;


INY #4

LDA #$00		;
LDY #$02		;
JSL $81B7B3	;
RTS		;

